@echo off
REM Created by Tyge Lovset Sept. 2016 (tycho)
REM flac.exe and lossywav.exe are released under the GPL license (see gpl.txt)
REM

setlocal
set h=0
if "%~1"=="" set h=1
if "%~1"=="-h" set h=1
if "%~1"=="--help" set h=1
if %h%==1 (
  echo lossyflac v1.2. Convert between WAV/FLAC and lossy.flac - by tycho 2016
  echo Resulting lossy.flac and flac files store tags and picture art from input.
  echo.
  echo Usage: %~nx0 inputfile... [lossywav-options] [@ flac-options]
  echo.
  echo Examples
  echo  " %~n0 *.flac -q P @ -8                      ' create lossy.flacs in same dir
  echo  " %~n0 *.wav -o lwdir                        ' create lossy.flac files in lwdir
  echo  " %~n0 *.flac -o lwdir -C -f                 ' create lossy+corr. flac files 
  echo  " %~n0 lwdir\*.lossy.flac -M -o flacdir      ' reconstruct flac files into flacdir
  echo  " %~n0 - -C -o song.lossy.flac < song.wav    ' stream to a lossy+corr. flac file 
  echo  " %~n0 song1.lossy.flac song2.lossy.flac -M  ' reconstruct two flac files
  goto final
)

REM Use tools (flac, metaflac, lossywav) in %PATH%:
set dir=
REM Use tools in same folder as lossyflac.cmd:
REM set "dir=%~dp0"

set "workdir=%TEMP%\lflac_%RANDOM%%RANDOM%"
mkdir "%workdir%"
set files=
set ok=0
:files_loop
  if "%~1"=="" goto files_done
  if "%~1"=="@" goto files_done
  set "f=%~1"
  if not "%~1"=="-" if "%f:~0,1%"=="-" goto files_done
  set files=%files% "%~1"
  set ok=1
  shift& goto files_loop
:files_done

if %ok%==0 (
  echo ERROR: a filename or "-" must be specified as first argument.
  goto final
)

set args=
:args_loop
  if not "%~1"=="" (set args=%args% "%~1"& shift& goto args_loop)
REM done

REM echo files: %files%, args: %args%
for %%F in (%files%) do call :single_file "%%~F" %args%
rmdir /s /q "%workdir%"
:final
endlocal
goto :eof


:single_file
setlocal
set lw_opts=
set fl_opts=
set "input=%~1"
set "iname=%~n1"
for %%F in ("%iname%") do set "ibname=%%~nF"
set "idir=%~dp1"
set "outdir=%~dp1"
set "oname=%~n1"
set output=
set force=
set operation=NORMAL
set flacInput=0
if /I "%~x1"==".flac" set flacInput=1
if /I "%~x1"==".fla" set flacInput=1

shift
:lw_loop
  shift
  if "%~0"=="" goto start
  if "%~0"=="@" goto fl_loop
  if "%~0"=="-f" (set force=-f& goto lw_loop)
  if "%~0"=="--force" (set force=-f& goto lw_loop)
  if "%~0"=="-C" set operation=CORRECTION
  if "%~0"=="--correction" set operation=CORRECTION
  if "%~0"=="-M" set operation=MERGE
  if "%~0"=="--merge" set operation=MERGE
  if "%~0"=="-o" (
    if exist "%~1\" (set "outdir=%~1") else (set "outdir=%~dp1"& set "oname=%~n1"& set "output=%~1")
    shift& goto lw_loop
  )
  if "%~0"=="--outdir" (set "outdir=%~1"& shift& goto lw_loop)
  set "lw_opts=%lw_opts% %0"
goto lw_loop
:fl_loop
  shift
  if "%~0"=="" goto start
  if "%~0"=="-f" (set force=-f& goto fl_loop)
  if "%~0"=="--force" (set force=-f& goto fl_loop)
  if "%~0"=="-o" (set "outdir=%~dp1"& set "oname=%~n1"& set "output=%~1"& shift& goto fl_loop)  
  if "%~0"=="--output-name" (set "outdir=%~dp1"& set "oname=%~n1"& set "output=%~1"& shift& goto fl_loop)
  set "fl_opts=%fl_opts% %0"
goto fl_loop

:start
if "%outdir:~-1%"=="\" set "outdir=%outdir:~0,-1%"
if not exist "%outdir%\" (
  echo ERROR: Output folder "%outdir%\" does not exist. Skipping "%input%".
  goto finish
)
for %%F in ("%oname%") do if /I "%%~xF"==".lossy" set "oname=%%~nF"
if "%input%"=="-" set "iname=%oname%"
if "%output%"=="" (
  if %operation%==MERGE (set "output=%outdir%\%oname%.flac") else (set "output=%outdir%\%oname%.lossy.flac")
)
REM call :debug
call :%operation%
:finish
endlocal
goto :eof


:NORMAL
if %flacInput%==1 (
  "%dir%flac.exe" -d "%input%" -o - --silent | "%dir%lossyWAV.exe" - %lw_opts% --stdout --stdinname "%oname%.wav" | "%dir%flac.exe" - -o "%output%" -b 512 --silent %fl_opts% %force%
  call :tags "%input%" "%output%"
) else (
  "%dir%lossyWAV.exe" "%input%" %lw_opts% --stdout --stdinname "%oname%.wav" | "%dir%flac.exe" - -o "%output%" -b 512 --silent %fl_opts% %force%
)
goto :eof


:CORRECTION
if "%force%"=="" if exist "%output%" (
  echo ERROR: output file "%output%" already exists. Use -f to override.
  goto :eof
)
if %flacInput%==1 (
  "%dir%flac.exe" "%input%" -d -o - --silent | "%dir%lossyWAV.exe" - %lw_opts% --stdinname "%oname%.wav" --outdir "%workdir%" --force
) else (
  "%dir%lossyWAV.exe" "%input%" %lw_opts% --stdinname "%oname%.wav" --outdir "%workdir%" --force
)
"%dir%flac.exe" "%workdir%\%oname%.lossy.wav" --totally-silent --delete-input-file --keep-foreign-metadata -b 512 %fl_opts% --force -o "%output%"
"%dir%flac.exe" "%workdir%\%oname%.lwcdf.wav" --totally-silent --delete-input-file --keep-foreign-metadata %fl_opts% --force -o "%outdir%\%oname%.lwcdf.flac"
if %flacInput%==1 call :tags "%input%" "%output%" "%outdir%\%oname%.lwcdf.flac"
goto :eof


:MERGE
if %flacInput%==1 (
  if "%force%"=="" if exist "%output%" (
    echo ERROR: output file "%output%" already exists. Use -f to override.
    goto :eof
  )
  "%dir%flac.exe" "%input%" -d --totally-silent --keep-foreign-metadata -o "%workdir%\%oname%.lossy.wav" --force
  "%dir%flac.exe" "%idir%%ibname%.lwcdf.flac" -d --totally-silent --keep-foreign-metadata -o "%workdir%\%oname%.lwcdf.wav" --force

  "%dir%lossyWAV.exe" "%workdir%\%oname%.lossy.wav" %lw_opts% --outdir "%workdir%" --force
  del "%workdir%\%oname%.lossy.wav" "%workdir%\%oname%.lwcdf.wav"
  "%dir%flac.exe" "%workdir%\%oname%.wav" --delete-input-file %fl_opts% --force -o "%output%"
  call :tags "%input%" "%output%"
) else (
  echo ERROR: lossyflac merges flac'ed lossyWAV files only. Use lossyWAV directly for wav files.
)
goto :eof


:tags
set tag="%workdir%\%oname%_tag.txt"
set pic="%workdir%\%oname%_pic.jpg"
"%dir%metaflac.exe" --export-tags-to=%tag% --export-picture-to=%pic% %1 2>nul
if exist %pic% (
  "%dir%metaflac.exe" --import-tags-from=%tag% --import-picture-from=%pic% %2
  REM Tag the correction file too. Overkill?
  if not "%~3"=="" "%dir%metaflac.exe" --import-tags-from=%tag% %3
  del %tag% %pic%
) else (
  "%dir%metaflac.exe" --import-tags-from=%tag% %2 %3
  del %tag%
)
goto :eof


:debug
echo iname: "%iname%"
echo ibname: "%ibname%"
echo idir: "%idir%"
echo oname: "%oname%"
echo outdir: "%outdir%"
echo flacInput: %flacInput%
echo operation: %operation%
echo lw_opts: %lw_opts%
echo fl_opts: %fl_opts%
echo.
goto :eof
